function Truss_PostProcessing
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% some useful MatLab postprocessing functions for trusses
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%input data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%define folder name

foldername = 'DYN_L_block_truss';
%foldername = 'DYN_rope_truss';
%foldername = 'DYN_rope_truss_flexible';
%foldername = 'DYN_rope_truss_fine_mesh';
%foldername = 'DYN_trussed_frame_elastic';


%incremental output
incrm=20;
% print out snapshots (YES=1; NO=0)
snap_label=1;
%snapshots to be printed out
snapshots = [20 120 160 200]; 
% generate video (YES=1; NO=0)
video_label=0;
%plot nodes label (YES=1; NO=0)
label=0;
%2D or 3D plot
dim=3;
%xy, xz, yz or xyz
if dim==2
   plane=1; %1:xz; 2:yz; 3:xy 
   %frame limits for video (in 2D)
   xmin=-120;xmax=120;ymin=-120;ymax=20;
   %xmin=-10;xmax=130;ymin=-10;ymax=130;
   zmin=-15;zmax=15; %dummy values
else
   plane=1; %dummy variable
   %frame limits for video (in 3D)
   xmin=-6;xmax=9;ymin=-6;ymax=13;zmin=-8;zmax=8;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%perform actions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
perform_tasks(foldername,snapshots,label,dim,plane,...
              xmin,xmax,ymin,ymax,zmin,zmax,incrm,snap_label,video_label)
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%List of functions used above
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function perform_tasks(foldername,snapshots,label,dim,plane,...
         xmin,xmax,ymin,ymax,zmin,zmax,incrm,snap_label,video_label)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot relevant diagrams
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('   job name: "%s" \n', foldername);
%add directory to path
[folder,max_incrm]=add_path(foldername,incrm);

% FIGURE 1: plot energy evolution (figure 1)
energy_plot(folder,foldername,max_incrm*incrm);

% FIGUREs 2 and 3: plot (time vs coordinate & coordinate vs force) of a given degree of
% freedom based on _FLAGOUT.TXT (figures 2 and 3)
flagout(foldername);

% FIGURE 4: plot increment snapshot (figure 4)
plot_truss_snapshot(foldername,max_incrm*incrm,label,dim,plane);

% export PDF diagrams 
if snap_label==1
   print_snaphots(folder,foldername,snapshots,label,dim,plane);
end

% generates video
if video_label==1
   generate_video(xmin,xmax,ymin,ymax,zmin,zmax,max_incrm,incrm,dim,plane,folder,foldername);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

function energy_plot(folder,foldername,i)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot energy evolution based on a given output file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%define file name
filename  =  [foldername '-increment_' num2str(i) '-RESTART.MAT'];
%load file name
load(filename);

%plot energy evolution
dt=CON.DYNAMICS.dt;
t=dt*[1:size(GEOM.Energy,2)];
h=figure;movegui(gcf,'northwest');
plot(t,GEOM.Energy,   'bo-',t,GEOM.IntEnergy,'rs--',t,GEOM.ExtEnergy,'kd:',t,GEOM.Kinetic,  'gx-.',...
     'LineWidth', 2, 'MarkerIndices', 1:5:length(t));
legend('Total Energy','Internal Energy','External Work','Kinetic Energy');
xlabel('time');ylabel('Energy');
grid on

%send to PDF file
if isunix()
    dirsep =  '/';    
else   
    dirsep =  '\';
end
Fig_name=[folder dirsep 'Energy_plot'];
PDF_plot(Fig_name);

end

function flagout(foldername)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot time history of a given degree of freedom based on _FLAGOUT.TXT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%plot time history of a given degree of freedom (from _FLAGOUT.TXT)
filename2 = [foldername '-OUTPUT_FLAGOUT.TXT'];
fid = fopen(filename2,'r+');
format = ['%g %g %g %g'];
info = fscanf(fid,format,[4,inf]);
fclose(fid);
incr=info(1,1:end);
x   =info(2,1:end);
f   =info(3,1:end);

%Increment vs Coordinate
h=figure;movegui(gcf,'north');
plot(incr,x,'bo-','LineWidth', 2, 'MarkerIndices', 1:1:length(x));
xlabel('Time increment');ylabel('Coordinate');
grid on;

%Coordinate vs Force applied
h=figure;movegui(gcf,'northeast');
plot(x,f,'rs-','LineWidth', 2, 'MarkerIndices', 1:1:length(x));
xlabel('Coordinate');ylabel('Force');
grid on;
end

function plot_truss_snapshot(foldername,i,label,dim,plane)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot deformed shape of 2D/3D truss based on a given output file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%define file name
filename  =  [foldername '-increment_' num2str(1*i) '-RESTART.MAT'];
%plot truss structure
plot_truss_structure(filename,label,dim,plane,1)
end

function plot_truss_structure(filename,label,dim,plane,visible)

    %load file name
    load(filename);
    
    %plot deformed shape
    h=figure;
    if visible 
        set(h,'Visible', 'on');
    else
        set(h,'Visible', 'off');
    end
    movegui(gcf,'south');
    for e=1:FEM.mesh.nelem
        i=FEM.mesh.connectivity(1,e);
        j=FEM.mesh.connectivity(2,e);
        x=[GEOM.x(1,i) GEOM.x(1,j)];
        y=[GEOM.x(2,i) GEOM.x(2,j)];
        z=[GEOM.x(3,i) GEOM.x(3,j)];
        plot_truss_element(x,y,z,dim,plane);
        hold on;
    end
    grid on;
    
    %label nodes
    if label==1
        plot_nodal_labels(GEOM,dim,plane);
    end
    
    %scale of figure
    axis equal
    
    %x, y and z labels
    if dim==2
        %1:xz; 2:yz; 3:xy
        if plane==1
            xlabel('x-axis');ylabel('z-axis');
        elseif plane==2
            xlabel('y-axis');ylabel('z-axis');
        else
            xlabel('x-axis');ylabel('y-axis');
        end
    elseif dim==3
        xlabel('x-axis');ylabel('y-axis');zlabel('z-axis');
    end
    %no grey background
    set(gcf,'color','w')
    


end

function print_snaphots(folder,foldername,list,label,dim,plane)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% print deformed shape of 2D/3D truss based on a given output file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('   Generating PDF figures\n');
%option 1: one figure per PDF file
for j=1:size(list,2)
    %define increment
    snap=list(j);
    %define file name
    filename  =  [foldername '-increment_' num2str(1*snap) '-RESTART.MAT'];
    %plot figure
    plot_truss_structure(filename,label,dim,plane,0)
    %send to PDF file
    if isunix()
        dirsep =  '/';
    else
        dirsep =  '\';
    end
    Fig_name=[folder dirsep 'Snapshot_' num2str(snap)];
    PDF_plot(Fig_name);
end
end

function plot_truss_element(x,y,z,dim,plane)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot truss element 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
if dim==2
    %1:xz; 2:yz; 3:xy
    if plane==1
        %plot(x,z,'bo-');
        plot(x,z,'b-');
    elseif plane==2
        %plot(y,z,'bo-');
        plot(y,z,'b-');
    else
        %plot(x,y,'bo-');
        plot(x,y,'b-');
    end
else
    %plot3(x,y,z,'bo-');
    plot3(x,y,z,'b-');
end
end

function plot_nodal_labels(GEOM,dim,plane)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot node labels
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:GEOM.npoin
    x=GEOM.x(1,i);
    y=GEOM.x(2,i);
    z=GEOM.x(3,i);
    if dim==2
        %1:xz; 2:yz; 3:xy
        if plane==1
            text(x,z,['\bullet' num2str(i)],'Fontsize',8)
        elseif plane==2
            text(y,z,['\bullet' num2str(i)],'Fontsize',8)
        else
            text(x,y,['\bullet' num2str(i)],'Fontsize',8)
        end
    elseif dim==3
        text(x,y,z,['\bullet' num2str(i)],'Fontsize',8)
    end
end    
end

function generate_video(xmin,xmax,ymin,ymax,zmin,zmax,n,step,dim,plane,folder,foldername)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%generates a video of truss
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('   Generating video\n');

%initialise avi loop
if isunix()
    dirsep =  '/';    
else   
    dirsep =  '\';
end
writerObj = VideoWriter([folder dirsep foldername '.avi']); 
open(writerObj);
for k=1:n
    filename  =  [foldername '-increment_' num2str(step*k) '-RESTART.MAT'];
    load(filename);
    
    %extract geometry
    if k==1
       X=GEOM.x0(:,:);
    else
       X=GEOM.x(:,:); 
    end
    
    %create figure and add snapshot
    h=figure;set(h,'Visible', 'off');
    for e=1:FEM.mesh.nelem
        i=FEM.mesh.connectivity(1,e);
        j=FEM.mesh.connectivity(2,e);
        x=[X(1,i) X(1,j)];
        y=[X(2,i) X(2,j)];
        z=[X(3,i) X(3,j)];
        plot_truss_element(x,y,z,dim,plane);
        hold on;
    end
    grid on;
    if dim==2
        axis([xmin xmax ymin ymax]);
    else
        axis([xmin xmax ymin ymax zmin zmax]);
    end
    set(gcf,'PaperUnits','inches');
    set(gcf,'PaperPositionMode','auto');
    set(gcf,'PaperSize',[5.5 4.5]);

    F = getframe;
    writeVideo(writerObj,F)
            
end
close(writerObj);
end

function [folder,last]=add_path(foldername,delta)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% add relevant directory to path
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%move to relevant directory
if isunix()
    dirsep =  '/';    
else   
    dirsep =  '\';
end
%current file
basedir = mfilename('fullpath');
%current folder
basedir = fileparts(basedir);
%folder with relevant example
folder=[basedir dirsep 'job_folder' dirsep foldername];
%add folder to path
addpath(folder)

%count files to be read
i = 0;
reading = 1;
while reading
    i = i+1; 
    filename  =  [foldername '-increment_' num2str(delta*i) '-RESTART.MAT'];
    if exist(filename,'file')
    else
        last = i-1;
        break;
    end
end
end

function PDF_plot(Fig_name)
set(gcf,'PaperUnits','inches');
set(gcf,'PaperPositionMode','auto');
set(gcf,'PaperSize',[5.5 4.5]);
print(gcf, '-dpdf', '-r1000', Fig_name,'-bestfit');
%print(gcf, '-dpng', '-r1000', Fig_name);
%print(gcf, '-djpeg', '-r1000', Fig_name);
end



