%--------------------------------------------------------------------------
% Leapfrog time integrator
%--------------------------------------------------------------------------
function LeapFrog(PRO,FEM,GEOM,QUADRATURE,BC,MAT,LOAD,...
                                  CON,CONSTANT,GLOBAL,PLAST,KINEMATICS)
                              

%--------------------------------------------------------------------------                              
% Size of the time step
%--------------------------------------------------------------------------                              
dt         =  CON.DYNAMICS.dt;
%--------------------------------------------------------------------------                              
% Compute lumped mass matrix
%--------------------------------------------------------------------------                              
[GLOBAL.Mass,Global.InvMass]    =  lump_mass_matrix(GLOBAL.Mass,GEOM.npoin*GEOM.ndime);                              
%--------------------------------------------------------------------------                              
% Set up initial conditions (if the problem is started from scratch)
%--------------------------------------------------------------------------                              
if ~PRO.rest
   %-----------------------------------------------------------------------
   % Initialise acceleration a0 to zero
   %-----------------------------------------------------------------------
   GEOM.acceleration        =  zeros(GEOM.ndime,GEOM.npoin);
   %-----------------------------------------------------------------------
   % Compute initial acceleration a0
   %-----------------------------------------------------------------------
   GEOM.acceleration(BC.freedof) = -Global.InvMass(BC.freedof).*(GLOBAL.T_int(BC.freedof) - GLOBAL.external_load(BC.freedof));      
   %-----------------------------------------------------------------------
   % Initialise time, time step 
   %-----------------------------------------------------------------------
   CON.DYNAMICS.time    =  0;
   CON.incrm  =  0;
   CON.niter  =  1;
   %-----------------------------------------------------------------------
   % Initialised load time factor 
   %-----------------------------------------------------------------------
   CON.xlamb  =  0;
end
%--------------------------------------------------------------------------                              
% Start time loop 
%--------------------------------------------------------------------------
while (CON.DYNAMICS.tmax-CON.DYNAMICS.time) > 1.0e-10
    %----------------------------------------------------------------------
    % Update time
    %----------------------------------------------------------------------
    CON.DYNAMICS.time  =  CON.DYNAMICS.time + dt;
    CON.incrm  =  CON.incrm +  1;
    %----------------------------------------------------------------------
    % Compute time load increment 
    %----------------------------------------------------------------------
    [CON.xlamb,CON.dlamb]  =  TimeLoadIncrement(CON.DYNAMICS.time,CON.xlamb,CON.DYNAMICS);
    %----------------------------------------------------------------------
    % Compute velocity v_n+1/2
    %----------------------------------------------------------------------
    GEOM.velocity      =  GEOM.velocity + GEOM.acceleration*dt/2; 
    %----------------------------------------------------------------------
    % Update fixed dofs  
    %----------------------------------------------------------------------
    GEOM.x = update_prescribed_displacements(BC.dofprescribed,...
                   GEOM.x0,GEOM.x,CON.xlamb,BC.presc_displacement);                                                           
    %----------------------------------------------------------------------
    % Update free dofs x_n+1  
    %----------------------------------------------------------------------               
    GEOM.x(BC.freedof)  =  GEOM.x(BC.freedof) + GEOM.velocity(BC.freedof)*dt;
    %--------------------------------------------------------------------
    % Update nodal forces (excluding pressure) and gravity. 
    %--------------------------------------------------------------------
    [GLOBAL.Residual,GLOBAL.external_load] = ...
                     external_force_update(GLOBAL.nominal_external_load,... 
                           GLOBAL.Residual,GLOBAL.external_load,CON.dlamb);
    %----------------------------------------------------------------------
    % Compute equivalent nodal forces and assembles residual force, 
    % excluding pressure contributions.
    %----------------------------------------------------------------------
    [GLOBAL,updated_PLAST] = explicit_residual_assembly(CON.xlamb,...
         GEOM,MAT,FEM,GLOBAL,CONSTANT,QUADRATURE.element,PLAST,KINEMATICS);        
    %----------------------------------------------------------------------
    % Update nodal forces due to pressure. 
    %----------------------------------------------------------------------
    if LOAD.n_pressure_loads
       GLOBAL = explicit_pressure_load_assembly(GEOM,FEM,GLOBAL,...
                                 LOAD,QUADRATURE.boundary,CON.xlamb);                              
    end
    %----------------------------------------------------------------------
    % Update acceleration a_n+1
    %----------------------------------------------------------------------
    GEOM.acceleration(BC.freedof) = -Global.InvMass(BC.freedof).*(GLOBAL.T_int(BC.freedof) - GLOBAL.external_load(BC.freedof));                                                          
    %----------------------------------------------------------------------
    % Update velocity v_n+1
    %----------------------------------------------------------------------
    GEOM.velocity(BC.freedof)  =  GEOM.velocity(BC.freedof) + GEOM.acceleration(BC.freedof)*dt/2;               
    %----------------------------------------------------------------------
    % Compute total energy.
    %----------------------------------------------------------------------
    [GEOM.Energy(CON.incrm),...
        GEOM.Kinetic(CON.incrm),...
        GEOM.IntEnergy(CON.incrm),...
        GEOM.ExtEnergy(CON.incrm)]  = TotalEnergyComputation(GEOM,MAT,FEM,GLOBAL,...
                                         QUADRATURE.element,KINEMATICS,CON);
    %----------------------------------------------------------------------
    % Compute reactions and update and save internal variables. 
    %----------------------------------------------------------------------
    GLOBAL.Reactions = GLOBAL.Residual(BC.fixdof) + GLOBAL.external_load(BC.fixdof);    
    PLAST            = save_output(updated_PLAST,PRO,FEM,GEOM,QUADRATURE,BC,...
                            MAT,LOAD,CON,CONSTANT,GLOBAL,PLAST,KINEMATICS);     
    %----------------------------------------------------------------------
    % Print time step. 
    %----------------------------------------------------------------------
    fprintf(['Step: ','%3i\n'],CON.incrm);        
end

fprintf(' Normal end of PROGRAM flagshyp. \n');


end