%--------------------------------------------------------------------------
% Alpha Method time integrator
%--------------------------------------------------------------------------
function MidPoint(PRO,FEM,GEOM,QUADRATURE,BC,MAT,LOAD,...
                                  CON,CONSTANT,GLOBAL,PLAST,KINEMATICS)
                              
                              
                              
%--------------------------------------------------------------------------                              
% Size of the time step
%--------------------------------------------------------------------------                              
dt         =  CON.DYNAMICS.dt;
if ~PRO.rest
   %-----------------------------------------------------------------------
   % Initialise acceleration a0 to zero
   %-----------------------------------------------------------------------
   GEOM.acceleration    =  zeros(GEOM.ndime,GEOM.npoin);
   %-----------------------------------------------------------------------
   % Initialise time, time step 
   %-----------------------------------------------------------------------
   CON.DYNAMICS.time    =  0;
   CON.incrm  =  0;
   %-----------------------------------------------------------------------
   % Initialised load time factor 
   %-----------------------------------------------------------------------
   CON.xlamb  =  0;
end
%--------------------------------------------------------------------------                              
% Start time loop
%--------------------------------------------------------------------------
%while (CON.DYNAMICS.time<CON.DYNAMICS.tmax)
while (CON.DYNAMICS.tmax-CON.DYNAMICS.time) > 1.0e-10  
    %----------------------------------------------------------------------
    % Update time
    %----------------------------------------------------------------------
    time_old               =  CON.DYNAMICS.time;    
    CON.DYNAMICS.time       =  CON.DYNAMICS.time + CON.DYNAMICS.dt;
    CON.incrm  =  CON.incrm + 1;
    %----------------------------------------------------------------------
    % Compute time load increment
    %----------------------------------------------------------------------
    timehalf               =  0.5*(CON.DYNAMICS.time + time_old);
    [CON.xlamb,CON.dlamb]  =  TimeLoadIncrement(timehalf,CON.xlamb,CON.DYNAMICS);
    %----------------------------------------------------------------------
    % Update fixed dofs  
    %----------------------------------------------------------------------
    GEOM.x = update_prescribed_displacements(BC.dofprescribed,...
                   GEOM.x0,GEOM.x,CON.xlamb,BC.presc_displacement);                                                       
    %----------------------------------------------------------------------
    % Velocities and geometry for previous time step (with updated Dirichlet)
    %----------------------------------------------------------------------
    xn         =  GEOM.x;
    vn         =  GEOM.velocity;
    %----------------------------------------------------------------------
    % Predictor step for mid-point method. 
    %----------------------------------------------------------------------
    GEOM.x      =  xn + dt/2*(vn + dt/2*GEOM.acceleration);
    GEOM.velocity    =  vn + dt*GEOM.acceleration;               
    %----------------------------------------------------------------------
    % Update nodal forces (excluding pressure) and gravity. 
    %----------------------------------------------------------------------
    [GLOBAL.Residual,GLOBAL.external_load] = ...
                     external_force_update(GLOBAL.nominal_external_load,...
                           GLOBAL.Residual,GLOBAL.external_load,CON.dlamb);
    %----------------------------------------------------------------------
    % Compute equivalent nodal forces and assembles residual force, 
    % excluding pressure contributions.
    %----------------------------------------------------------------------
    [GLOBAL,updated_PLAST] = residual_and_stiffness_assembly(CON.xlamb,...
         GEOM,MAT,FEM,GLOBAL,CONSTANT,QUADRATURE.element,PLAST,KINEMATICS);     
    %----------------------------------------------------------------------
    % Update nodal forces due to pressure. 
    %----------------------------------------------------------------------
    if LOAD.n_pressure_loads
       GLOBAL = pressure_load_and_stiffness_assembly(GEOM,MAT,FEM,...
                              GLOBAL,LOAD,QUADRATURE.boundary,CON.xlamb);
    end
    %----------------------------------------------------------------------
    % Newton-Raphson iteration.
    %----------------------------------------------------------------------
    CON.niter = 0;
    rnorm = 2*CON.cnorm;
    %----------------------------------------------------------------------
    % Modified mass matrix and residual.
    %----------------------------------------------------------------------        
    GLOBAL.K         =  GLOBAL.Mass + dt/2*0 + dt^2/4*GLOBAL.K;
    GLOBAL.Residual  =  GLOBAL.Mass*GEOM.acceleration(:) + 0*GEOM.velocity(:) + GLOBAL.Residual;
    while((rnorm > CON.cnorm) && (CON.niter < CON.miter))
        CON.niter = CON.niter + 1;
        %------------------------------------------------------------------
        % Solve for iterative accelerations (da).
        %------------------------------------------------------------------        
        da      = linear_solver(GLOBAL.K,-GLOBAL.Residual,BC.fixdof);
        %------------------------------------------------------------------
        % Corrector step for mid-point method.
        %------------------------------------------------------------------
        GEOM.acceleration(BC.freedof)  =  GEOM.acceleration(BC.freedof) + da;
        GEOM.velocity(BC.freedof)      =  GEOM.velocity(BC.freedof)     + dt*da;      
        GEOM.x(BC.freedof)             =  GEOM.x(BC.freedof)       + dt^2/4*da;
        %------------------------------------------------------------------        
        % -Recompute equivalent nodal forces and assembles residual force,
        %  excluding pressure contributions.
        % -Recompute and assembles tangent stiffness matrix components,
        %  excluding pressure contributions.
        %------------------------------------------------------------------
        [GLOBAL,updated_PLAST] = residual_and_stiffness_assembly(CON.xlamb,...
            GEOM,MAT,FEM,GLOBAL,CONSTANT,QUADRATURE.element,PLAST,KINEMATICS);
        %------------------------------------------------------------------
        % Update nodal forces due to pressure.
        %------------------------------------------------------------------
        if LOAD.n_pressure_loads
            GLOBAL = pressure_load_and_stiffness_assembly(GEOM,MAT,FEM,...
                GLOBAL,LOAD,QUADRATURE.boundary,CON.xlamb);
        end
        %------------------------------------------------------------------
        % Modified mass matrix and residual.
        %------------------------------------------------------------------        
        GLOBAL.K         =  GLOBAL.Mass + dt/2*0 + dt^2/4*GLOBAL.K;
        GLOBAL.Residual  =  GLOBAL.Mass*GEOM.acceleration(:) + 0*GEOM.velocity(:) + GLOBAL.Residual;
        %------------------------------------------------------------------
        % Check for equilibrium convergence.
        %------------------------------------------------------------------
        [rnorm,GLOBAL] = check_residual_norm(CON,BC,GLOBAL,BC.freedof);
        %------------------------------------------------------------------
        % Break iteration before residual gets unrealistic (e.g. NaN).
        %------------------------------------------------------------------
        if (abs(rnorm)>1e7 || isnan(rnorm))
            CON.niter=CON.miter;
            break;
        end 
    end 
    %----------------------------------------------------------------------
    % If convergence not achieved opt to restart from previous results or
    % terminate. 
    %----------------------------------------------------------------------
    if( CON.niter >= CON.miter)
        terminate = input(['Solution not converged. Do you want to '...
            'terminate the program (y/n) ?: ' ' \n'],'s');
        if( strcmp(deblank(terminate),deblank('n')) || strcmp(deblank(terminate),deblank('N')))
            load(PRO.internal_restartfile_name);
            fprintf(['Restart from previous step by decreasing '...
                'the time step (dt) to half its '...
                'initally fixed value. \n']);
            CON.DYNAMICS.dt = CON.DYNAMICS.dt/2;
        else
            fclose('all');
            error('Program terminated by user');
        end
    else
        %------------------------------------------------------------------
        % If convergence was achieved, update correct 
        % geometry at the end of the time step.
        %------------------------------------------------------------------
        GEOM.x           =  2*GEOM.x   - xn;
        %------------------------------------------------------------------
        % Compute total energy.
        %------------------------------------------------------------------        
        [GEOM.Energy(CON.incrm),...
         GEOM.Kinetic(CON.incrm),...
         GEOM.IntEnergy(CON.incrm),...
         GEOM.ExtEnergy(CON.incrm)]  = TotalEnergyComputation(GEOM,MAT,FEM,GLOBAL,...
                                   QUADRATURE.element,KINEMATICS,CON);
        %------------------------------------------------------------------
        % If convergence was achieved, update and save internal variables.
        %------------------------------------------------------------------
        PLAST = save_output(updated_PLAST,PRO,FEM,GEOM,QUADRATURE,BC,...
            MAT,LOAD,CON,CONSTANT,GLOBAL,PLAST,KINEMATICS);
    end
end
fprintf(' Normal end of PROGRAM flagshyp. \n');
